@@ -42,6 +42,8 @@ typedef struct _ValaCCodeAssignment ValaCCodeAssignment;
 typedef struct _ValaCCodeAssignmentClass ValaCCodeAssignmentClass;
 typedef struct _ValaCCodeAssignmentPrivate ValaCCodeAssignmentPrivate;
 
+#define VALA_TYPE_CCODE_ASSIGNMENT_OPERATOR (vala_ccode_assignment_operator_get_type ())
+
 /**
  * Represents an assignment expression in the C code.
  */
@@ -49,9 +51,11 @@ struct _ValaCCodeAssignment {
 	ValaCCodeExpression parent_instance;
 	ValaCCodeAssignmentPrivate * priv;
 };
+
 struct _ValaCCodeAssignmentClass {
 	ValaCCodeExpressionClass parent_class;
 };
+
 typedef enum  {
 	VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE,
 	VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_OR,
@@ -66,12 +70,18 @@ typedef enum  {
 	VALA_CCODE_ASSIGNMENT_OPERATOR_SHIFT_RIGHT
 } ValaCCodeAssignmentOperator;
 
+
+ValaCCodeAssignment* vala_ccode_assignment_construct (GType object_type, ValaCCodeExpression* l, ValaCCodeExpression* r, ValaCCodeAssignmentOperator op);
 ValaCCodeAssignment* vala_ccode_assignment_new (ValaCCodeExpression* l, ValaCCodeExpression* r, ValaCCodeAssignmentOperator op);
 ValaCCodeExpression* vala_ccode_assignment_get_left (ValaCCodeAssignment* self);
+void vala_ccode_assignment_set_left (ValaCCodeAssignment* self, ValaCCodeExpression* value);
 ValaCCodeAssignmentOperator vala_ccode_assignment_get_operator (ValaCCodeAssignment* self);
 void vala_ccode_assignment_set_operator (ValaCCodeAssignment* self, ValaCCodeAssignmentOperator value);
 ValaCCodeExpression* vala_ccode_assignment_get_right (ValaCCodeAssignment* self);
+void vala_ccode_assignment_set_right (ValaCCodeAssignment* self, ValaCCodeExpression* value);
 GType vala_ccode_assignment_get_type (void);
+GType vala_ccode_assignment_operator_get_type (void);
+
 
 G_END_DECLS
 
