@@ -657,15 +657,15 @@ static void vala_ccode_member_access_module_real_visit_member_access (ValaCCodeM
 								if (vala_property_get_binding (prop) == MEMBER_BINDING_INSTANCE) {
 									vala_ccode_function_call_add_argument (ccall, pub_inst);
 								}
-								/* Property acesses to real struct types are handeled different to other properties.
-								 They are returned as out parameter.*/
+								/* Property access to real struct types is handled differently
+								 The value is returned by out parameter*/
 								if (vala_data_type_is_real_struct_type (vala_property_get_property_type (base_property))) {
 									ValaCCodeCommaExpression* ccomma;
 									ValaLocalVariable* temp_var;
 									ValaCCodeIdentifier* ctemp;
 									ValaCCodeUnaryExpression* _tmp120;
 									ccomma = vala_ccode_comma_expression_new ();
-									temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_property_get_property_type (base_property), TRUE, NULL);
+									temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_property_accessor_get_value_type (vala_property_get_get_accessor (base_property)), TRUE, NULL);
 									ctemp = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) temp_var));
 									gee_collection_add ((GeeCollection*) ((ValaCCodeBaseModule*) self)->temp_vars, temp_var);
 									_tmp120 = NULL;
@@ -687,214 +687,200 @@ static void vala_ccode_member_access_module_real_visit_member_access (ValaCCodeM
 								ValaCCodeIdentifier* _tmp121;
 								ValaCCodeFunctionCall* _tmp122;
 								ValaCCodeFunctionCall* ccall;
-								ValaCCodeIdentifier* _tmp123;
-								ValaCCodeFunctionCall* _tmp124;
-								ValaCCodeFunctionCall* ccast;
-								ValaCCodeConstant* _tmp125;
-								ValaDataType* temp_type;
+								ValaCCodeConstant* _tmp123;
+								ValaCCodeCommaExpression* ccomma;
 								ValaLocalVariable* temp_var;
 								ValaCCodeIdentifier* ctemp;
-								ValaCCodeUnaryExpression* _tmp126;
-								ValaCCodeConstant* _tmp127;
-								ValaCCodeCommaExpression* ccomma;
+								ValaCCodeUnaryExpression* _tmp124;
+								ValaCCodeConstant* _tmp125;
 								_tmp121 = NULL;
 								_tmp122 = NULL;
 								ccall = (_tmp122 = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp121 = vala_ccode_identifier_new ("g_object_get"))), (_tmp121 == NULL) ? NULL : (_tmp121 = (vala_ccode_node_unref (_tmp121), NULL)), _tmp122);
-								_tmp123 = NULL;
-								_tmp124 = NULL;
-								ccast = (_tmp124 = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp123 = vala_ccode_identifier_new ("G_OBJECT"))), (_tmp123 == NULL) ? NULL : (_tmp123 = (vala_ccode_node_unref (_tmp123), NULL)), _tmp124);
-								vala_ccode_function_call_add_argument (ccast, pub_inst);
-								vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) ccast);
+								vala_ccode_function_call_add_argument (ccall, pub_inst);
 								/* property name is second argument of g_object_get*/
-								_tmp125 = NULL;
-								vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp125 = vala_property_get_canonical_cconstant (prop)));
-								(_tmp125 == NULL) ? NULL : (_tmp125 = (vala_ccode_node_unref (_tmp125), NULL));
-								/* g_object_get always returns owned values*/
-								temp_type = vala_data_type_copy (vala_expression_get_value_type ((ValaExpression*) expr));
-								vala_data_type_set_value_owned (temp_type, TRUE);
-								/* we need a temporary variable to save the property value*/
-								temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_expression_get_value_type ((ValaExpression*) expr), TRUE, NULL);
-								gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
-								if (vala_ccode_base_module_requires_destroy ((ValaCCodeBaseModule*) self, temp_type)) {
-									gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_ref_vars, 0, temp_var);
-								}
-								ctemp = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) temp_var));
-								_tmp126 = NULL;
-								vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp126 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) ctemp)));
-								(_tmp126 == NULL) ? NULL : (_tmp126 = (vala_ccode_node_unref (_tmp126), NULL));
-								_tmp127 = NULL;
-								vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp127 = vala_ccode_constant_new ("NULL")));
-								(_tmp127 == NULL) ? NULL : (_tmp127 = (vala_ccode_node_unref (_tmp127), NULL));
+								_tmp123 = NULL;
+								vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp123 = vala_property_get_canonical_cconstant (prop)));
+								(_tmp123 == NULL) ? NULL : (_tmp123 = (vala_ccode_node_unref (_tmp123), NULL));
+								/* g_object_get always returns owned values
+								 therefore, property getters of properties
+								 without accessor methods need to be marked as owned*/
 								ccomma = vala_ccode_comma_expression_new ();
+								temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_expression_get_value_type ((ValaExpression*) expr), TRUE, NULL);
+								ctemp = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) temp_var));
+								gee_collection_add ((GeeCollection*) ((ValaCCodeBaseModule*) self)->temp_vars, temp_var);
+								_tmp124 = NULL;
+								vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp124 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) ctemp)));
+								(_tmp124 == NULL) ? NULL : (_tmp124 = (vala_ccode_node_unref (_tmp124), NULL));
+								_tmp125 = NULL;
+								vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp125 = vala_ccode_constant_new ("NULL")));
+								(_tmp125 == NULL) ? NULL : (_tmp125 = (vala_ccode_node_unref (_tmp125), NULL));
 								vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) ccall);
 								vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) ctemp);
 								vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccomma);
 								(ccall == NULL) ? NULL : (ccall = (vala_ccode_node_unref (ccall), NULL));
-								(ccast == NULL) ? NULL : (ccast = (vala_ccode_node_unref (ccast), NULL));
-								(temp_type == NULL) ? NULL : (temp_type = (vala_code_node_unref (temp_type), NULL));
+								(ccomma == NULL) ? NULL : (ccomma = (vala_ccode_node_unref (ccomma), NULL));
 								(temp_var == NULL) ? NULL : (temp_var = (vala_code_node_unref (temp_var), NULL));
 								(ctemp == NULL) ? NULL : (ctemp = (vala_ccode_node_unref (ctemp), NULL));
-								(ccomma == NULL) ? NULL : (ccomma = (vala_ccode_node_unref (ccomma), NULL));
 							}
 						}
 						(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
 					} else {
 						if (VALA_IS_ENUM_VALUE (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
-							ValaEnumValue* _tmp128;
+							ValaEnumValue* _tmp126;
 							ValaEnumValue* ev;
-							ValaCCodeConstant* _tmp130;
-							char* _tmp129;
+							ValaCCodeConstant* _tmp128;
+							char* _tmp127;
+							_tmp126 = NULL;
+							ev = (_tmp126 = VALA_ENUM_VALUE (vala_expression_get_symbol_reference ((ValaExpression*) expr)), (_tmp126 == NULL) ? NULL : vala_code_node_ref (_tmp126));
 							_tmp128 = NULL;
-							ev = (_tmp128 = VALA_ENUM_VALUE (vala_expression_get_symbol_reference ((ValaExpression*) expr)), (_tmp128 == NULL) ? NULL : vala_code_node_ref (_tmp128));
-							_tmp130 = NULL;
-							_tmp129 = NULL;
-							vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp130 = vala_ccode_constant_new (_tmp129 = vala_enum_value_get_cname (ev))));
-							(_tmp130 == NULL) ? NULL : (_tmp130 = (vala_ccode_node_unref (_tmp130), NULL));
-							_tmp129 = (g_free (_tmp129), NULL);
+							_tmp127 = NULL;
+							vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp128 = vala_ccode_constant_new (_tmp127 = vala_enum_value_get_cname (ev))));
+							(_tmp128 == NULL) ? NULL : (_tmp128 = (vala_ccode_node_unref (_tmp128), NULL));
+							_tmp127 = (g_free (_tmp127), NULL);
 							(ev == NULL) ? NULL : (ev = (vala_code_node_unref (ev), NULL));
 						} else {
 							if (VALA_IS_LOCAL_VARIABLE (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
-								ValaLocalVariable* _tmp131;
+								ValaLocalVariable* _tmp129;
 								ValaLocalVariable* local;
-								ValaCCodeExpression* _tmp132;
-								_tmp131 = NULL;
-								local = (_tmp131 = VALA_LOCAL_VARIABLE (vala_expression_get_symbol_reference ((ValaExpression*) expr)), (_tmp131 == NULL) ? NULL : vala_code_node_ref (_tmp131));
-								_tmp132 = NULL;
-								vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp132 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) local))));
-								(_tmp132 == NULL) ? NULL : (_tmp132 = (vala_ccode_node_unref (_tmp132), NULL));
+								ValaCCodeExpression* _tmp130;
+								_tmp129 = NULL;
+								local = (_tmp129 = VALA_LOCAL_VARIABLE (vala_expression_get_symbol_reference ((ValaExpression*) expr)), (_tmp129 == NULL) ? NULL : vala_code_node_ref (_tmp129));
+								_tmp130 = NULL;
+								vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp130 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) local))));
+								(_tmp130 == NULL) ? NULL : (_tmp130 = (vala_ccode_node_unref (_tmp130), NULL));
 								(local == NULL) ? NULL : (local = (vala_code_node_unref (local), NULL));
 							} else {
 								if (VALA_IS_FORMAL_PARAMETER (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
-									ValaFormalParameter* _tmp133;
+									ValaFormalParameter* _tmp131;
 									ValaFormalParameter* p;
-									_tmp133 = NULL;
-									p = (_tmp133 = VALA_FORMAL_PARAMETER (vala_expression_get_symbol_reference ((ValaExpression*) expr)), (_tmp133 == NULL) ? NULL : vala_code_node_ref (_tmp133));
+									_tmp131 = NULL;
+									p = (_tmp131 = VALA_FORMAL_PARAMETER (vala_expression_get_symbol_reference ((ValaExpression*) expr)), (_tmp131 == NULL) ? NULL : vala_code_node_ref (_tmp131));
 									if (_vala_strcmp0 (vala_symbol_get_name ((ValaSymbol*) p), "this") == 0) {
-										gboolean _tmp134;
-										_tmp134 = FALSE;
+										gboolean _tmp132;
+										_tmp132 = FALSE;
 										if (((ValaCCodeBaseModule*) self)->current_method != NULL) {
-											_tmp134 = vala_method_get_coroutine (((ValaCCodeBaseModule*) self)->current_method);
+											_tmp132 = vala_method_get_coroutine (((ValaCCodeBaseModule*) self)->current_method);
 										} else {
-											_tmp134 = FALSE;
+											_tmp132 = FALSE;
 										}
-										if (_tmp134) {
-											ValaCCodeExpression* _tmp135;
+										if (_tmp132) {
+											ValaCCodeExpression* _tmp133;
 											/* use closure*/
-											_tmp135 = NULL;
-											vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp135 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "self")));
-											(_tmp135 == NULL) ? NULL : (_tmp135 = (vala_ccode_node_unref (_tmp135), NULL));
+											_tmp133 = NULL;
+											vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp133 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "self")));
+											(_tmp133 == NULL) ? NULL : (_tmp133 = (vala_ccode_node_unref (_tmp133), NULL));
 										} else {
-											ValaStruct* _tmp137;
-											ValaTypeSymbol* _tmp136;
+											ValaStruct* _tmp135;
+											ValaTypeSymbol* _tmp134;
 											ValaStruct* st;
-											gboolean _tmp138;
-											_tmp137 = NULL;
-											_tmp136 = NULL;
-											st = (_tmp137 = (_tmp136 = ((ValaCCodeBaseModule*) self)->current_type_symbol, VALA_IS_STRUCT (_tmp136) ? ((ValaStruct*) _tmp136) : NULL), (_tmp137 == NULL) ? NULL : vala_code_node_ref (_tmp137));
-											_tmp138 = FALSE;
+											gboolean _tmp136;
+											_tmp135 = NULL;
+											_tmp134 = NULL;
+											st = (_tmp135 = (_tmp134 = ((ValaCCodeBaseModule*) self)->current_type_symbol, VALA_IS_STRUCT (_tmp134) ? ((ValaStruct*) _tmp134) : NULL), (_tmp135 == NULL) ? NULL : vala_code_node_ref (_tmp135));
+											_tmp136 = FALSE;
 											if (st != NULL) {
-												_tmp138 = !vala_struct_is_simple_type (st);
+												_tmp136 = !vala_struct_is_simple_type (st);
 											} else {
-												_tmp138 = FALSE;
+												_tmp136 = FALSE;
 											}
-											if (_tmp138) {
-												ValaCCodeIdentifier* _tmp139;
-												_tmp139 = NULL;
-												vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp139 = vala_ccode_identifier_new ("(*self)")));
-												(_tmp139 == NULL) ? NULL : (_tmp139 = (vala_ccode_node_unref (_tmp139), NULL));
+											if (_tmp136) {
+												ValaCCodeIdentifier* _tmp137;
+												_tmp137 = NULL;
+												vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp137 = vala_ccode_identifier_new ("(*self)")));
+												(_tmp137 == NULL) ? NULL : (_tmp137 = (vala_ccode_node_unref (_tmp137), NULL));
 											} else {
-												ValaCCodeIdentifier* _tmp140;
-												_tmp140 = NULL;
-												vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp140 = vala_ccode_identifier_new ("self")));
-												(_tmp140 == NULL) ? NULL : (_tmp140 = (vala_ccode_node_unref (_tmp140), NULL));
+												ValaCCodeIdentifier* _tmp138;
+												_tmp138 = NULL;
+												vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp138 = vala_ccode_identifier_new ("self")));
+												(_tmp138 == NULL) ? NULL : (_tmp138 = (vala_ccode_node_unref (_tmp138), NULL));
 											}
 											(st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL));
 										}
 									} else {
-										gboolean _tmp141;
-										_tmp141 = FALSE;
+										gboolean _tmp139;
+										_tmp139 = FALSE;
 										if (((ValaCCodeBaseModule*) self)->current_method != NULL) {
-											_tmp141 = vala_method_get_coroutine (((ValaCCodeBaseModule*) self)->current_method);
+											_tmp139 = vala_method_get_coroutine (((ValaCCodeBaseModule*) self)->current_method);
 										} else {
-											_tmp141 = FALSE;
+											_tmp139 = FALSE;
 										}
-										if (_tmp141) {
-											ValaCCodeExpression* _tmp142;
+										if (_tmp139) {
+											ValaCCodeExpression* _tmp140;
 											/* use closure*/
-											_tmp142 = NULL;
-											vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp142 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) p))));
-											(_tmp142 == NULL) ? NULL : (_tmp142 = (vala_ccode_node_unref (_tmp142), NULL));
+											_tmp140 = NULL;
+											vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp140 = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) p))));
+											(_tmp140 == NULL) ? NULL : (_tmp140 = (vala_ccode_node_unref (_tmp140), NULL));
 										} else {
-											ValaStruct* _tmp144;
-											ValaTypeSymbol* _tmp143;
+											ValaStruct* _tmp142;
+											ValaTypeSymbol* _tmp141;
 											ValaStruct* type_as_struct;
-											gboolean _tmp145;
-											_tmp144 = NULL;
-											_tmp143 = NULL;
-											type_as_struct = (_tmp144 = (_tmp143 = vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (p)), VALA_IS_STRUCT (_tmp143) ? ((ValaStruct*) _tmp143) : NULL), (_tmp144 == NULL) ? NULL : vala_code_node_ref (_tmp144));
-											_tmp145 = FALSE;
+											gboolean _tmp143;
+											_tmp142 = NULL;
+											_tmp141 = NULL;
+											type_as_struct = (_tmp142 = (_tmp141 = vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (p)), VALA_IS_STRUCT (_tmp141) ? ((ValaStruct*) _tmp141) : NULL), (_tmp142 == NULL) ? NULL : vala_code_node_ref (_tmp142));
+											_tmp143 = FALSE;
 											if (vala_formal_parameter_get_direction (p) != VALA_PARAMETER_DIRECTION_IN) {
-												_tmp145 = TRUE;
+												_tmp143 = TRUE;
 											} else {
-												gboolean _tmp146;
-												gboolean _tmp147;
-												_tmp146 = FALSE;
-												_tmp147 = FALSE;
+												gboolean _tmp144;
+												gboolean _tmp145;
+												_tmp144 = FALSE;
+												_tmp145 = FALSE;
 												if (type_as_struct != NULL) {
-													_tmp147 = !vala_struct_is_simple_type (type_as_struct);
+													_tmp145 = !vala_struct_is_simple_type (type_as_struct);
 												} else {
-													_tmp147 = FALSE;
+													_tmp145 = FALSE;
 												}
-												if (_tmp147) {
-													_tmp146 = !vala_data_type_get_nullable (vala_formal_parameter_get_parameter_type (p));
+												if (_tmp145) {
+													_tmp144 = !vala_data_type_get_nullable (vala_formal_parameter_get_parameter_type (p));
 												} else {
-													_tmp146 = FALSE;
+													_tmp144 = FALSE;
 												}
-												_tmp145 = (_tmp146);
+												_tmp143 = (_tmp144);
 											}
-											if (_tmp145) {
-												ValaCCodeIdentifier* _tmp149;
-												char* _tmp148;
-												_tmp149 = NULL;
-												_tmp148 = NULL;
-												vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp149 = vala_ccode_identifier_new (_tmp148 = g_strdup_printf ("(*%s)", vala_symbol_get_name ((ValaSymbol*) p)))));
-												(_tmp149 == NULL) ? NULL : (_tmp149 = (vala_ccode_node_unref (_tmp149), NULL));
-												_tmp148 = (g_free (_tmp148), NULL);
+											if (_tmp143) {
+												ValaCCodeIdentifier* _tmp147;
+												char* _tmp146;
+												_tmp147 = NULL;
+												_tmp146 = NULL;
+												vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp147 = vala_ccode_identifier_new (_tmp146 = g_strdup_printf ("(*%s)", vala_symbol_get_name ((ValaSymbol*) p)))));
+												(_tmp147 == NULL) ? NULL : (_tmp147 = (vala_ccode_node_unref (_tmp147), NULL));
+												_tmp146 = (g_free (_tmp146), NULL);
 											} else {
+												gboolean _tmp148;
+												gboolean _tmp149;
 												gboolean _tmp150;
-												gboolean _tmp151;
-												gboolean _tmp152;
+												_tmp148 = FALSE;
+												_tmp149 = FALSE;
 												_tmp150 = FALSE;
-												_tmp151 = FALSE;
-												_tmp152 = FALSE;
 												if (((ValaCCodeBaseModule*) self)->current_property_accessor != NULL) {
-													_tmp152 = vala_property_accessor_get_writable (((ValaCCodeBaseModule*) self)->current_property_accessor);
-												} else {
-													_tmp152 = FALSE;
-												}
-												if (_tmp152) {
-													_tmp151 = vala_property_accessor_get_value_parameter (((ValaCCodeBaseModule*) self)->current_property_accessor) == p;
-												} else {
-													_tmp151 = FALSE;
-												}
-												if (_tmp151) {
-													_tmp150 = vala_data_type_is_real_struct_type (vala_property_get_property_type (vala_property_accessor_get_prop (((ValaCCodeBaseModule*) self)->current_property_accessor)));
+													_tmp150 = vala_property_accessor_get_writable (((ValaCCodeBaseModule*) self)->current_property_accessor);
 												} else {
 													_tmp150 = FALSE;
 												}
+												if (_tmp150) {
+													_tmp149 = vala_property_accessor_get_value_parameter (((ValaCCodeBaseModule*) self)->current_property_accessor) == p;
+												} else {
+													_tmp149 = FALSE;
+												}
+												if (_tmp149) {
+													_tmp148 = vala_data_type_is_real_struct_type (vala_property_get_property_type (vala_property_accessor_get_prop (((ValaCCodeBaseModule*) self)->current_property_accessor)));
+												} else {
+													_tmp148 = FALSE;
+												}
 												/* Property setters of non simple structs shall replace all occurences
 												 of the "value" formal parameter with a dereferencing version of that
 												 parameter.*/
-												if (_tmp150) {
-													ValaCCodeIdentifier* _tmp153;
-													_tmp153 = NULL;
-													vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp153 = vala_ccode_identifier_new ("(*value)")));
-													(_tmp153 == NULL) ? NULL : (_tmp153 = (vala_ccode_node_unref (_tmp153), NULL));
+												if (_tmp148) {
+													ValaCCodeIdentifier* _tmp151;
+													_tmp151 = NULL;
+													vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp151 = vala_ccode_identifier_new ("(*value)")));
+													(_tmp151 == NULL) ? NULL : (_tmp151 = (vala_ccode_node_unref (_tmp151), NULL));
 												} else {
-													ValaCCodeIdentifier* _tmp154;
-													_tmp154 = NULL;
-													vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp154 = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) p))));
-													(_tmp154 == NULL) ? NULL : (_tmp154 = (vala_ccode_node_unref (_tmp154), NULL));
+													ValaCCodeIdentifier* _tmp152;
+													_tmp152 = NULL;
+													vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp152 = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) p))));
+													(_tmp152 == NULL) ? NULL : (_tmp152 = (vala_ccode_node_unref (_tmp152), NULL));
 												}
 											}
 											(type_as_struct == NULL) ? NULL : (type_as_struct = (vala_code_node_unref (type_as_struct), NULL));
