@@ -1,6 +1,6 @@
 /* valacodecontext.vala
  *
- * Copyright (C) 2006-2008  Jürg Billeter
+ * Copyright (C) 2006-2009  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,9 +28,9 @@
 #include <gee/collection.h>
 #include <glib/gstdio.h>
 #include <config.h>
-#include <vala/valamethod.h>
 #include <vala/valanamespace.h>
 #include <vala/valasourcereference.h>
+#include <vala/valamethod.h>
 #include <vala/valacodevisitor.h>
 #include <vala/valasourcefile.h>
 #include <vala/valasourcefilecycle.h>
@@ -45,7 +45,8 @@ struct _ValaCodeContextPrivate {
 	gboolean _memory_management;
 	gboolean _assert;
 	gboolean _checking;
-	gboolean _non_null;
+	gboolean _deprecated;
+	gboolean _experimental;
 	gboolean _non_null_experimental;
 	gboolean _dbus_transformation;
 	gboolean _ccode_only;
@@ -76,6 +77,7 @@ static ValaSourceFile* vala_code_context_find_cycle_head (ValaCodeContext* self,
 static void vala_code_context_visit (ValaCodeContext* self, ValaSourceFile* file, GeeList* chain);
 static gpointer vala_code_context_parent_class = NULL;
 static void vala_code_context_finalize (ValaCodeContext* obj);
+static gint _vala_array_length (gpointer array);
 
 
 
@@ -507,11 +509,11 @@ static void vala_code_context_visit (ValaCodeContext* self, ValaSourceFile* file
 }
 
 
-char* vala_code_context_get_package_path (ValaCodeContext* self, const char* pkg, int vapi_directories_length1, char** vapi_directories) {
+char* vala_code_context_get_package_path (ValaCodeContext* self, const char* pkg, char** vapi_directories, int vapi_directories_length1) {
 	char* basename;
 	char* filename;
 	char* _tmp3;
-	char* _tmp8;
+	char* _tmp9;
 	g_return_val_if_fail (self != NULL, NULL);
 	g_return_val_if_fail (pkg != NULL, NULL);
 	basename = g_strdup_printf ("%s.vapi", pkg);
@@ -523,7 +525,7 @@ char* vala_code_context_get_package_path (ValaCodeContext* self, const char* pkg
 			int vapidir_it;
 			vapidir_collection = vapi_directories;
 			vapidir_collection_length1 = vapi_directories_length1;
-			for (vapidir_it = 0; ((vapi_directories_length1 != -1) && (vapidir_it < vapi_directories_length1)) || ((vapi_directories_length1 == -1) && (vapidir_collection[vapidir_it] != NULL)); vapidir_it = vapidir_it + 1) {
+			for (vapidir_it = 0; vapidir_it < vapi_directories_length1; vapidir_it = vapidir_it + 1) {
 				const char* _tmp2;
 				char* vapidir;
 				_tmp2 = NULL;
@@ -550,31 +552,33 @@ char* vala_code_context_get_package_path (ValaCodeContext* self, const char* pkg
 		return (_tmp4 = filename, basename = (g_free (basename), NULL), _tmp4);
 	}
 	{
+		char** _tmp5;
 		char** vapidir_collection;
 		int vapidir_collection_length1;
-		char** vapidir_it;
-		vapidir_collection = g_get_system_data_dirs ();
-		vapidir_collection_length1 = -1;
-		for (vapidir_it = vapidir_collection; *vapidir_it != NULL; vapidir_it = vapidir_it + 1) {
-			const char* _tmp7;
+		int vapidir_it;
+		_tmp5 = NULL;
+		vapidir_collection = _tmp5 = g_get_system_data_dirs ();
+		vapidir_collection_length1 = _vala_array_length (_tmp5);
+		for (vapidir_it = 0; vapidir_it < _vala_array_length (_tmp5); vapidir_it = vapidir_it + 1) {
+			const char* _tmp8;
 			char* vapidir;
-			_tmp7 = NULL;
-			vapidir = (_tmp7 = *vapidir_it, (_tmp7 == NULL) ? NULL : g_strdup (_tmp7));
+			_tmp8 = NULL;
+			vapidir = (_tmp8 = vapidir_collection[vapidir_it], (_tmp8 == NULL) ? NULL : g_strdup (_tmp8));
 			{
-				char* _tmp5;
-				_tmp5 = NULL;
-				filename = (_tmp5 = g_build_filename (vapidir, "vala/vapi", basename, NULL), filename = (g_free (filename), NULL), _tmp5);
+				char* _tmp6;
+				_tmp6 = NULL;
+				filename = (_tmp6 = g_build_filename (vapidir, "vala/vapi", basename, NULL), filename = (g_free (filename), NULL), _tmp6);
 				if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
-					char* _tmp6;
-					_tmp6 = NULL;
-					return (_tmp6 = filename, vapidir = (g_free (vapidir), NULL), basename = (g_free (basename), NULL), _tmp6);
+					char* _tmp7;
+					_tmp7 = NULL;
+					return (_tmp7 = filename, vapidir = (g_free (vapidir), NULL), basename = (g_free (basename), NULL), _tmp7);
 				}
 				vapidir = (g_free (vapidir), NULL);
 			}
 		}
 	}
-	_tmp8 = NULL;
-	return (_tmp8 = NULL, basename = (g_free (basename), NULL), filename = (g_free (filename), NULL), _tmp8);
+	_tmp9 = NULL;
+	return (_tmp9 = NULL, basename = (g_free (basename), NULL), filename = (g_free (filename), NULL), _tmp9);
 }
 
 
@@ -630,15 +634,27 @@ void vala_code_context_set_checking (ValaCodeContext* self, gboolean value) {
 }
 
 
-gboolean vala_code_context_get_non_null (ValaCodeContext* self) {
+gboolean vala_code_context_get_deprecated (ValaCodeContext* self) {
 	g_return_val_if_fail (self != NULL, FALSE);
-	return self->priv->_non_null;
+	return self->priv->_deprecated;
 }
 
 
-void vala_code_context_set_non_null (ValaCodeContext* self, gboolean value) {
+void vala_code_context_set_deprecated (ValaCodeContext* self, gboolean value) {
 	g_return_if_fail (self != NULL);
-	self->priv->_non_null = value;
+	self->priv->_deprecated = value;
+}
+
+
+gboolean vala_code_context_get_experimental (ValaCodeContext* self) {
+	g_return_val_if_fail (self != NULL, FALSE);
+	return self->priv->_experimental;
+}
+
+
+void vala_code_context_set_experimental (ValaCodeContext* self, gboolean value) {
+	g_return_if_fail (self != NULL);
+	self->priv->_experimental = value;
 }
 
 
@@ -1026,5 +1042,17 @@ void vala_code_context_unref (gpointer instance) {
 }
 
 
+static gint _vala_array_length (gpointer array) {
+	int length;
+	length = 0;
+	if (array) {
+		while (((gpointer*) array)[length]) {
+			length++;
+		}
+	}
+	return length;
+}
+
+
 
 
