@@ -1,6 +1,6 @@
 /* valaerrordomain.vala
  *
- * Copyright (C) 2008  Jürg Billeter
+ * Copyright (C) 2008-2009  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,8 +23,9 @@
 #include <vala/valaerrordomain.h>
 #include <gee/arraylist.h>
 #include <gee/collection.h>
+#include <gee/iterable.h>
+#include <gee/iterator.h>
 #include <gee/readonlylist.h>
-#include <vala/valacodevisitor.h>
 #include <vala/valaerrorcode.h>
 #include <vala/valamethod.h>
 #include <vala/valasourcereference.h>
@@ -34,9 +35,11 @@
 #include <vala/valareport.h>
 #include <vala/valamember.h>
 #include <vala/valaformalparameter.h>
-#include <vala/valavaluetype.h>
-#include <vala/valadatatype.h>
+#include <vala/valasemanticanalyzer.h>
 #include <vala/valaattribute.h>
+#include <vala/valaerrortype.h>
+#include <vala/valadatatype.h>
+#include <vala/valacodevisitor.h>
 
 
 
@@ -68,9 +71,11 @@ static char* vala_error_domain_real_get_type_id (ValaTypeSymbol* base);
 static char* vala_error_domain_real_get_marshaller_type_name (ValaTypeSymbol* base);
 static char* vala_error_domain_real_get_get_value_function (ValaTypeSymbol* base);
 static char* vala_error_domain_real_get_set_value_function (ValaTypeSymbol* base);
+static gboolean vala_error_domain_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer);
 static gpointer vala_error_domain_parent_class = NULL;
 static void vala_error_domain_finalize (ValaCodeNode* obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
+static gint _vala_array_length (gpointer array);
 static int _vala_strcmp0 (const char * str1, const char * str2);
 
 
@@ -85,8 +90,7 @@ static int _vala_strcmp0 (const char * str1, const char * str2);
 ValaErrorDomain* vala_error_domain_construct (GType object_type, const char* name, ValaSourceReference* source_reference) {
 	ValaErrorDomain* self;
 	g_return_val_if_fail (name != NULL, NULL);
-	self = ((ValaErrorDomain*) (g_type_create_instance (object_type)));
-	self = ((ValaErrorDomain*) (vala_typesymbol_construct (object_type, name, source_reference)));
+	self = (ValaErrorDomain*) vala_typesymbol_construct (object_type, name, source_reference);
 	return self;
 }
 
@@ -104,8 +108,8 @@ ValaErrorDomain* vala_error_domain_new (const char* name, ValaSourceReference* s
 void vala_error_domain_add_code (ValaErrorDomain* self, ValaErrorCode* ecode) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (ecode != NULL);
-	gee_collection_add (((GeeCollection*) (self->priv->codes)), ecode);
-	vala_scope_add (vala_symbol_get_scope (((ValaSymbol*) (self))), vala_symbol_get_name (((ValaSymbol*) (ecode))), ((ValaSymbol*) (ecode)));
+	gee_collection_add ((GeeCollection*) self->priv->codes, ecode);
+	vala_scope_add (vala_symbol_get_scope ((ValaSymbol*) self), vala_symbol_get_name ((ValaSymbol*) ecode), (ValaSymbol*) ecode);
 }
 
 
@@ -118,22 +122,22 @@ void vala_error_domain_add_method (ValaErrorDomain* self, ValaMethod* m) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (m != NULL);
 	if (VALA_IS_CREATION_METHOD (m)) {
-		vala_report_error (vala_code_node_get_source_reference (((ValaCodeNode*) (m))), "construction methods may only be declared within classes and structs");
-		vala_code_node_set_error (((ValaCodeNode*) (m)), TRUE);
+		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) m), "construction methods may only be declared within classes and structs");
+		vala_code_node_set_error ((ValaCodeNode*) m, TRUE);
 		return;
 	}
 	if (vala_method_get_binding (m) == MEMBER_BINDING_INSTANCE) {
 		ValaFormalParameter* _tmp1;
-		ValaValueType* _tmp0;
+		ValaErrorType* _tmp0;
 		_tmp1 = NULL;
 		_tmp0 = NULL;
-		vala_method_set_this_parameter (m, (_tmp1 = vala_formal_parameter_new ("this", ((ValaDataType*) ((_tmp0 = vala_value_type_new (((ValaTypeSymbol*) (self)))))), NULL)));
-		(_tmp1 == NULL ? NULL : (_tmp1 = (vala_code_node_unref (_tmp1), NULL)));
-		(_tmp0 == NULL ? NULL : (_tmp0 = (vala_code_node_unref (_tmp0), NULL)));
-		vala_scope_add (vala_symbol_get_scope (((ValaSymbol*) (m))), vala_symbol_get_name (((ValaSymbol*) (vala_method_get_this_parameter (m)))), ((ValaSymbol*) (vala_method_get_this_parameter (m))));
+		vala_method_set_this_parameter (m, _tmp1 = vala_formal_parameter_new ("this", (ValaDataType*) (_tmp0 = vala_error_type_new (self, NULL, NULL)), NULL));
+		(_tmp1 == NULL) ? NULL : (_tmp1 = (vala_code_node_unref (_tmp1), NULL));
+		(_tmp0 == NULL) ? NULL : (_tmp0 = (vala_code_node_unref (_tmp0), NULL));
+		vala_scope_add (vala_symbol_get_scope ((ValaSymbol*) m), vala_symbol_get_name ((ValaSymbol*) vala_method_get_this_parameter (m)), (ValaSymbol*) vala_method_get_this_parameter (m));
 	}
-	gee_collection_add (((GeeCollection*) (self->priv->methods)), m);
-	vala_scope_add (vala_symbol_get_scope (((ValaSymbol*) (self))), vala_symbol_get_name (((ValaSymbol*) (m))), ((ValaSymbol*) (m)));
+	gee_collection_add ((GeeCollection*) self->priv->methods, m);
+	vala_scope_add (vala_symbol_get_scope ((ValaSymbol*) self), vala_symbol_get_name ((ValaSymbol*) m), (ValaSymbol*) m);
 }
 
 
@@ -144,7 +148,7 @@ void vala_error_domain_add_method (ValaErrorDomain* self, ValaMethod* m) {
  */
 GeeList* vala_error_domain_get_codes (ValaErrorDomain* self) {
 	g_return_val_if_fail (self != NULL, NULL);
-	return ((GeeList*) (gee_read_only_list_new (VALA_TYPE_ERROR_CODE, ((GBoxedCopyFunc) (vala_code_node_ref)), vala_code_node_unref, self->priv->codes)));
+	return (GeeList*) gee_read_only_list_new (VALA_TYPE_ERROR_CODE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, self->priv->codes);
 }
 
 
@@ -155,13 +159,13 @@ GeeList* vala_error_domain_get_codes (ValaErrorDomain* self) {
  */
 GeeList* vala_error_domain_get_methods (ValaErrorDomain* self) {
 	g_return_val_if_fail (self != NULL, NULL);
-	return ((GeeList*) (gee_read_only_list_new (VALA_TYPE_METHOD, ((GBoxedCopyFunc) (vala_code_node_ref)), vala_code_node_unref, self->priv->methods)));
+	return (GeeList*) gee_read_only_list_new (VALA_TYPE_METHOD, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, self->priv->methods);
 }
 
 
 static void vala_error_domain_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaErrorDomain * self;
-	self = ((ValaErrorDomain*) (base));
+	self = (ValaErrorDomain*) base;
 	g_return_if_fail (visitor != NULL);
 	vala_code_visitor_visit_error_domain (visitor, self);
 }
@@ -169,33 +173,29 @@ static void vala_error_domain_real_accept (ValaCodeNode* base, ValaCodeVisitor*
 
 static void vala_error_domain_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaErrorDomain * self;
-	self = ((ValaErrorDomain*) (base));
+	self = (ValaErrorDomain*) base;
 	g_return_if_fail (visitor != NULL);
 	{
-		GeeList* ecode_collection;
-		int ecode_it;
-		ecode_collection = self->priv->codes;
-		for (ecode_it = 0; ecode_it < gee_collection_get_size (GEE_COLLECTION (ecode_collection)); ecode_it = ecode_it + 1) {
+		GeeIterator* _ecode_it;
+		_ecode_it = gee_iterable_iterator ((GeeIterable*) self->priv->codes);
+		while (gee_iterator_next (_ecode_it)) {
 			ValaErrorCode* ecode;
-			ecode = ((ValaErrorCode*) (gee_list_get (GEE_LIST (ecode_collection), ecode_it)));
-			{
-				vala_code_node_accept (((ValaCodeNode*) (ecode)), visitor);
-				(ecode == NULL ? NULL : (ecode = (vala_code_node_unref (ecode), NULL)));
-			}
+			ecode = (ValaErrorCode*) gee_iterator_get (_ecode_it);
+			vala_code_node_accept ((ValaCodeNode*) ecode, visitor);
+			(ecode == NULL) ? NULL : (ecode = (vala_code_node_unref (ecode), NULL));
 		}
+		(_ecode_it == NULL) ? NULL : (_ecode_it = (gee_collection_object_unref (_ecode_it), NULL));
 	}
 	{
-		GeeList* m_collection;
-		int m_it;
-		m_collection = self->priv->methods;
-		for (m_it = 0; m_it < gee_collection_get_size (GEE_COLLECTION (m_collection)); m_it = m_it + 1) {
+		GeeIterator* _m_it;
+		_m_it = gee_iterable_iterator ((GeeIterable*) self->priv->methods);
+		while (gee_iterator_next (_m_it)) {
 			ValaMethod* m;
-			m = ((ValaMethod*) (gee_list_get (GEE_LIST (m_collection), m_it)));
-			{
-				vala_code_node_accept (((ValaCodeNode*) (m)), visitor);
-				(m == NULL ? NULL : (m = (vala_code_node_unref (m), NULL)));
-			}
+			m = (ValaMethod*) gee_iterator_get (_m_it);
+			vala_code_node_accept ((ValaCodeNode*) m, visitor);
+			(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
 		}
+		(_m_it == NULL) ? NULL : (_m_it = (gee_collection_object_unref (_m_it), NULL));
 	}
 }
 
@@ -203,34 +203,34 @@ static void vala_error_domain_real_accept_children (ValaCodeNode* base, ValaCode
 static char* vala_error_domain_real_get_cname (ValaTypeSymbol* base, gboolean const_type) {
 	ValaErrorDomain * self;
 	const char* _tmp2;
-	self = ((ValaErrorDomain*) (base));
+	self = (ValaErrorDomain*) base;
 	if (self->priv->cname == NULL) {
 		char* _tmp1;
 		char* _tmp0;
 		_tmp1 = NULL;
 		_tmp0 = NULL;
-		self->priv->cname = (_tmp1 = g_strdup_printf ("%s%s", (_tmp0 = vala_symbol_get_cprefix (vala_symbol_get_parent_symbol (((ValaSymbol*) (self))))), vala_symbol_get_name (((ValaSymbol*) (self)))), (self->priv->cname = (g_free (self->priv->cname), NULL)), _tmp1);
+		self->priv->cname = (_tmp1 = g_strdup_printf ("%s%s", _tmp0 = vala_symbol_get_cprefix (vala_symbol_get_parent_symbol ((ValaSymbol*) self)), vala_symbol_get_name ((ValaSymbol*) self)), self->priv->cname = (g_free (self->priv->cname), NULL), _tmp1);
 		_tmp0 = (g_free (_tmp0), NULL);
 	}
 	_tmp2 = NULL;
-	return (_tmp2 = self->priv->cname, (_tmp2 == NULL ? NULL : g_strdup (_tmp2)));
+	return (_tmp2 = self->priv->cname, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2));
 }
 
 
 static char* vala_error_domain_real_get_lower_case_cprefix (ValaSymbol* base) {
 	ValaErrorDomain * self;
 	const char* _tmp2;
-	self = ((ValaErrorDomain*) (base));
+	self = (ValaErrorDomain*) base;
 	if (self->priv->lower_case_cprefix == NULL) {
 		char* _tmp1;
 		char* _tmp0;
 		_tmp1 = NULL;
 		_tmp0 = NULL;
-		self->priv->lower_case_cprefix = (_tmp1 = g_strdup_printf ("%s_", (_tmp0 = vala_symbol_get_lower_case_cname (((ValaSymbol*) (self)), NULL))), (self->priv->lower_case_cprefix = (g_free (self->priv->lower_case_cprefix), NULL)), _tmp1);
+		self->priv->lower_case_cprefix = (_tmp1 = g_strdup_printf ("%s_", _tmp0 = vala_symbol_get_lower_case_cname ((ValaSymbol*) self, NULL)), self->priv->lower_case_cprefix = (g_free (self->priv->lower_case_cprefix), NULL), _tmp1);
 		_tmp0 = (g_free (_tmp0), NULL);
 	}
 	_tmp2 = NULL;
-	return (_tmp2 = self->priv->lower_case_cprefix, (_tmp2 == NULL ? NULL : g_strdup (_tmp2)));
+	return (_tmp2 = self->priv->lower_case_cprefix, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2));
 }
 
 
@@ -240,10 +240,10 @@ static char* vala_error_domain_get_lower_case_csuffix (ValaErrorDomain* self) {
 	if (self->priv->lower_case_csuffix == NULL) {
 		char* _tmp0;
 		_tmp0 = NULL;
-		self->priv->lower_case_csuffix = (_tmp0 = vala_symbol_camel_case_to_lower_case (vala_symbol_get_name (((ValaSymbol*) (self)))), (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL)), _tmp0);
+		self->priv->lower_case_csuffix = (_tmp0 = vala_symbol_camel_case_to_lower_case (vala_symbol_get_name ((ValaSymbol*) self)), self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL), _tmp0);
 	}
 	_tmp1 = NULL;
-	return (_tmp1 = self->priv->lower_case_csuffix, (_tmp1 == NULL ? NULL : g_strdup (_tmp1)));
+	return (_tmp1 = self->priv->lower_case_csuffix, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
 }
 
 
@@ -252,14 +252,14 @@ static char* vala_error_domain_real_get_lower_case_cname (ValaSymbol* base, cons
 	char* _tmp1;
 	char* _tmp0;
 	char* _tmp2;
-	self = ((ValaErrorDomain*) (base));
+	self = (ValaErrorDomain*) base;
 	if (infix == NULL) {
 		infix = "";
 	}
 	_tmp1 = NULL;
 	_tmp0 = NULL;
 	_tmp2 = NULL;
-	return (_tmp2 = g_strdup_printf ("%s%s%s", (_tmp0 = vala_symbol_get_lower_case_cprefix (vala_symbol_get_parent_symbol (((ValaSymbol*) (self))))), infix, (_tmp1 = vala_error_domain_get_lower_case_csuffix (self))), (_tmp1 = (g_free (_tmp1), NULL)), (_tmp0 = (g_free (_tmp0), NULL)), _tmp2);
+	return (_tmp2 = g_strdup_printf ("%s%s%s", _tmp0 = vala_symbol_get_lower_case_cprefix (vala_symbol_get_parent_symbol ((ValaSymbol*) self)), infix, _tmp1 = vala_error_domain_get_lower_case_csuffix (self)), _tmp1 = (g_free (_tmp1), NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp2);
 }
 
 
@@ -267,16 +267,16 @@ static char* vala_error_domain_real_get_upper_case_cname (ValaTypeSymbol* base,
 	ValaErrorDomain * self;
 	char* _tmp0;
 	char* _tmp1;
-	self = ((ValaErrorDomain*) (base));
+	self = (ValaErrorDomain*) base;
 	_tmp0 = NULL;
 	_tmp1 = NULL;
-	return (_tmp1 = g_utf8_strup ((_tmp0 = vala_symbol_get_lower_case_cname (((ValaSymbol*) (self)), infix)), -1), (_tmp0 = (g_free (_tmp0), NULL)), _tmp1);
+	return (_tmp1 = g_utf8_strup (_tmp0 = vala_symbol_get_lower_case_cname ((ValaSymbol*) self, infix), -1), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
 }
 
 
 static gboolean vala_error_domain_real_is_reference_type (ValaTypeSymbol* base) {
 	ValaErrorDomain * self;
-	self = ((ValaErrorDomain*) (base));
+	self = (ValaErrorDomain*) base;
 	return FALSE;
 }
 
@@ -288,7 +288,7 @@ static void vala_error_domain_set_cname (ValaErrorDomain* self, const char* cnam
 	g_return_if_fail (cname != NULL);
 	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->cname = (_tmp1 = (_tmp0 = cname, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->cname = (g_free (self->priv->cname), NULL)), _tmp1);
+	self->priv->cname = (_tmp1 = (_tmp0 = cname, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->priv->cname = (g_free (self->priv->cname), NULL), _tmp1);
 }
 
 
@@ -306,11 +306,11 @@ char* vala_error_domain_get_cprefix (ValaErrorDomain* self) {
 		char* _tmp0;
 		_tmp1 = NULL;
 		_tmp0 = NULL;
-		self->priv->cprefix = (_tmp1 = g_strdup_printf ("%s_", (_tmp0 = vala_typesymbol_get_upper_case_cname (((ValaTypeSymbol*) (self)), NULL))), (self->priv->cprefix = (g_free (self->priv->cprefix), NULL)), _tmp1);
+		self->priv->cprefix = (_tmp1 = g_strdup_printf ("%s_", _tmp0 = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) self, NULL)), self->priv->cprefix = (g_free (self->priv->cprefix), NULL), _tmp1);
 		_tmp0 = (g_free (_tmp0), NULL);
 	}
 	_tmp2 = NULL;
-	return (_tmp2 = self->priv->cprefix, (_tmp2 == NULL ? NULL : g_strdup (_tmp2)));
+	return (_tmp2 = self->priv->cprefix, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2));
 }
 
 
@@ -327,7 +327,7 @@ void vala_error_domain_set_cprefix (ValaErrorDomain* self, const char* cprefix)
 	g_return_if_fail (cprefix != NULL);
 	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->cprefix = (_tmp1 = (_tmp0 = cprefix, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->cprefix = (g_free (self->priv->cprefix), NULL)), _tmp1);
+	self->priv->cprefix = (_tmp1 = (_tmp0 = cprefix, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->priv->cprefix = (g_free (self->priv->cprefix), NULL), _tmp1);
 }
 
 
@@ -337,40 +337,42 @@ static void vala_error_domain_process_ccode_attribute (ValaErrorDomain* self, Va
 	if (vala_attribute_has_argument (a, "cname")) {
 		char* _tmp0;
 		_tmp0 = NULL;
-		vala_error_domain_set_cname (self, (_tmp0 = vala_attribute_get_string (a, "cname")));
+		vala_error_domain_set_cname (self, _tmp0 = vala_attribute_get_string (a, "cname"));
 		_tmp0 = (g_free (_tmp0), NULL);
 	}
 	if (vala_attribute_has_argument (a, "cprefix")) {
 		char* _tmp1;
 		_tmp1 = NULL;
-		vala_error_domain_set_cprefix (self, (_tmp1 = vala_attribute_get_string (a, "cprefix")));
+		vala_error_domain_set_cprefix (self, _tmp1 = vala_attribute_get_string (a, "cprefix"));
 		_tmp1 = (g_free (_tmp1), NULL);
 	}
 	if (vala_attribute_has_argument (a, "lower_case_csuffix")) {
 		char* _tmp2;
 		_tmp2 = NULL;
-		self->priv->lower_case_csuffix = (_tmp2 = vala_attribute_get_string (a, "lower_case_csuffix"), (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL)), _tmp2);
+		self->priv->lower_case_csuffix = (_tmp2 = vala_attribute_get_string (a, "lower_case_csuffix"), self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL), _tmp2);
 	}
 	if (vala_attribute_has_argument (a, "cheader_filename")) {
 		char* val;
 		val = vala_attribute_get_string (a, "cheader_filename");
 		{
+			char** _tmp3;
 			char** filename_collection;
 			int filename_collection_length1;
-			char** filename_it;
-			filename_collection = g_strsplit (val, ",", 0);
-			filename_collection_length1 = -1;
-			for (filename_it = filename_collection; *filename_it != NULL; filename_it = filename_it + 1) {
-				const char* _tmp3;
+			int filename_it;
+			_tmp3 = NULL;
+			filename_collection = _tmp3 = g_strsplit (val, ",", 0);
+			filename_collection_length1 = _vala_array_length (_tmp3);
+			for (filename_it = 0; filename_it < _vala_array_length (_tmp3); filename_it = filename_it + 1) {
+				const char* _tmp4;
 				char* filename;
-				_tmp3 = NULL;
-				filename = (_tmp3 = *filename_it, (_tmp3 == NULL ? NULL : g_strdup (_tmp3)));
+				_tmp4 = NULL;
+				filename = (_tmp4 = filename_collection[filename_it], (_tmp4 == NULL) ? NULL : g_strdup (_tmp4));
 				{
-					vala_typesymbol_add_cheader_filename (((ValaTypeSymbol*) (self)), filename);
+					vala_typesymbol_add_cheader_filename ((ValaTypeSymbol*) self, filename);
 					filename = (g_free (filename), NULL);
 				}
 			}
-			filename_collection = (_vala_array_free (filename_collection, filename_collection_length1, ((GDestroyNotify) (g_free))), NULL);
+			filename_collection = (_vala_array_free (filename_collection, filename_collection_length1, (GDestroyNotify) g_free), NULL);
 		}
 		val = (g_free (val), NULL);
 	}
@@ -385,17 +387,17 @@ void vala_error_domain_process_attributes (ValaErrorDomain* self) {
 	{
 		GList* a_collection;
 		GList* a_it;
-		a_collection = ((ValaCodeNode*) (self))->attributes;
+		a_collection = ((ValaCodeNode*) self)->attributes;
 		for (a_it = a_collection; a_it != NULL; a_it = a_it->next) {
 			ValaAttribute* _tmp0;
 			ValaAttribute* a;
 			_tmp0 = NULL;
-			a = (_tmp0 = ((ValaAttribute*) (a_it->data)), (_tmp0 == NULL ? NULL : vala_code_node_ref (_tmp0)));
+			a = (_tmp0 = (ValaAttribute*) a_it->data, (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
 			{
 				if (_vala_strcmp0 (vala_attribute_get_name (a), "CCode") == 0) {
 					vala_error_domain_process_ccode_attribute (self, a);
 				}
-				(a == NULL ? NULL : (a = (vala_code_node_unref (a), NULL)));
+				(a == NULL) ? NULL : (a = (vala_code_node_unref (a), NULL));
 			}
 		}
 	}
@@ -404,32 +406,67 @@ void vala_error_domain_process_attributes (ValaErrorDomain* self) {
 
 static char* vala_error_domain_real_get_type_id (ValaTypeSymbol* base) {
 	ValaErrorDomain * self;
-	self = ((ValaErrorDomain*) (base));
+	self = (ValaErrorDomain*) base;
 	return g_strdup ("G_TYPE_POINTER");
 }
 
 
 static char* vala_error_domain_real_get_marshaller_type_name (ValaTypeSymbol* base) {
 	ValaErrorDomain * self;
-	self = ((ValaErrorDomain*) (base));
+	self = (ValaErrorDomain*) base;
 	return g_strdup ("POINTER");
 }
 
 
 static char* vala_error_domain_real_get_get_value_function (ValaTypeSymbol* base) {
 	ValaErrorDomain * self;
-	self = ((ValaErrorDomain*) (base));
+	self = (ValaErrorDomain*) base;
 	return g_strdup ("g_value_get_pointer");
 }
 
 
 static char* vala_error_domain_real_get_set_value_function (ValaTypeSymbol* base) {
 	ValaErrorDomain * self;
-	self = ((ValaErrorDomain*) (base));
+	self = (ValaErrorDomain*) base;
 	return g_strdup ("g_value_set_pointer");
 }
 
 
+static gboolean vala_error_domain_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer) {
+	ValaErrorDomain * self;
+	self = (ValaErrorDomain*) base;
+	g_return_val_if_fail (analyzer != NULL, FALSE);
+	if (vala_code_node_get_checked ((ValaCodeNode*) self)) {
+		return !vala_code_node_get_error ((ValaCodeNode*) self);
+	}
+	vala_code_node_set_checked ((ValaCodeNode*) self, TRUE);
+	vala_error_domain_process_attributes (self);
+	{
+		GeeIterator* _ecode_it;
+		_ecode_it = gee_iterable_iterator ((GeeIterable*) self->priv->codes);
+		while (gee_iterator_next (_ecode_it)) {
+			ValaErrorCode* ecode;
+			ecode = (ValaErrorCode*) gee_iterator_get (_ecode_it);
+			vala_code_node_check ((ValaCodeNode*) ecode, analyzer);
+			(ecode == NULL) ? NULL : (ecode = (vala_code_node_unref (ecode), NULL));
+		}
+		(_ecode_it == NULL) ? NULL : (_ecode_it = (gee_collection_object_unref (_ecode_it), NULL));
+	}
+	{
+		GeeIterator* _m_it;
+		_m_it = gee_iterable_iterator ((GeeIterable*) self->priv->methods);
+		while (gee_iterator_next (_m_it)) {
+			ValaMethod* m;
+			m = (ValaMethod*) gee_iterator_get (_m_it);
+			vala_code_node_check ((ValaCodeNode*) m, analyzer);
+			(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
+		}
+		(_m_it == NULL) ? NULL : (_m_it = (gee_collection_object_unref (_m_it), NULL));
+	}
+	return !vala_code_node_get_error ((ValaCodeNode*) self);
+}
+
+
 static void vala_error_domain_class_init (ValaErrorDomainClass * klass) {
 	vala_error_domain_parent_class = g_type_class_peek_parent (klass);
 	VALA_CODE_NODE_CLASS (klass)->finalize = vala_error_domain_finalize;
@@ -445,21 +482,22 @@ static void vala_error_domain_class_init (ValaErrorDomainClass * klass) {
 	VALA_TYPESYMBOL_CLASS (klass)->get_marshaller_type_name = vala_error_domain_real_get_marshaller_type_name;
 	VALA_TYPESYMBOL_CLASS (klass)->get_get_value_function = vala_error_domain_real_get_get_value_function;
 	VALA_TYPESYMBOL_CLASS (klass)->get_set_value_function = vala_error_domain_real_get_set_value_function;
+	VALA_CODE_NODE_CLASS (klass)->check = vala_error_domain_real_check;
 }
 
 
 static void vala_error_domain_instance_init (ValaErrorDomain * self) {
 	self->priv = VALA_ERROR_DOMAIN_GET_PRIVATE (self);
-	self->priv->codes = ((GeeList*) (gee_array_list_new (VALA_TYPE_ERROR_CODE, ((GBoxedCopyFunc) (vala_code_node_ref)), vala_code_node_unref, g_direct_equal)));
-	self->priv->methods = ((GeeList*) (gee_array_list_new (VALA_TYPE_METHOD, ((GBoxedCopyFunc) (vala_code_node_ref)), vala_code_node_unref, g_direct_equal)));
+	self->priv->codes = (GeeList*) gee_array_list_new (VALA_TYPE_ERROR_CODE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal);
+	self->priv->methods = (GeeList*) gee_array_list_new (VALA_TYPE_METHOD, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal);
 }
 
 
 static void vala_error_domain_finalize (ValaCodeNode* obj) {
 	ValaErrorDomain * self;
 	self = VALA_ERROR_DOMAIN (obj);
-	(self->priv->codes == NULL ? NULL : (self->priv->codes = (gee_collection_object_unref (self->priv->codes), NULL)));
-	(self->priv->methods == NULL ? NULL : (self->priv->methods = (gee_collection_object_unref (self->priv->methods), NULL)));
+	(self->priv->codes == NULL) ? NULL : (self->priv->codes = (gee_collection_object_unref (self->priv->codes), NULL));
+	(self->priv->methods == NULL) ? NULL : (self->priv->methods = (gee_collection_object_unref (self->priv->methods), NULL));
 	self->priv->cname = (g_free (self->priv->cname), NULL);
 	self->priv->cprefix = (g_free (self->priv->cprefix), NULL);
 	self->priv->lower_case_cprefix = (g_free (self->priv->lower_case_cprefix), NULL);
@@ -479,28 +517,36 @@ GType vala_error_domain_get_type (void) {
 
 
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
-	if (array != NULL && destroy_func != NULL) {
+	if ((array != NULL) && (destroy_func != NULL)) {
 		int i;
-		if (array_length >= 0)
 		for (i = 0; i < array_length; i = i + 1) {
-			if (((gpointer*) (array))[i] != NULL)
-			destroy_func (((gpointer*) (array))[i]);
-		}
-		else
-		for (i = 0; ((gpointer*) (array))[i] != NULL; i = i + 1) {
-			destroy_func (((gpointer*) (array))[i]);
+			if (((gpointer*) array)[i] != NULL) {
+				destroy_func (((gpointer*) array)[i]);
+			}
 		}
 	}
 	g_free (array);
 }
 
 
+static gint _vala_array_length (gpointer array) {
+	int length;
+	length = 0;
+	if (array) {
+		while (((gpointer*) array)[length]) {
+			length++;
+		}
+	}
+	return length;
+}
+
+
 static int _vala_strcmp0 (const char * str1, const char * str2) {
 	if (str1 == NULL) {
 		return -(str1 != str2);
 	}
 	if (str2 == NULL) {
-		return (str1 != str2);
+		return str1 != str2;
 	}
 	return strcmp (str1, str2);
 }
