@@ -25,19 +25,15 @@
 #include <vala/valamethod.h>
 #include <gee/readonlylist.h>
 #include <gee/arraylist.h>
-#include <gee/collection.h>
 #include <gee/iterable.h>
 #include <gee/iterator.h>
+#include <gee/collection.h>
 #include <vala/valadatatype.h>
 #include <vala/valasourcereference.h>
 #include <vala/valaformalparameter.h>
 #include <vala/valablock.h>
 #include <vala/valasymbol.h>
 #include <vala/valaattribute.h>
-#include <vala/valascope.h>
-#include <vala/valaexpression.h>
-#include <vala/valalocalvariable.h>
-#include <vala/valacodevisitor.h>
 #include <vala/valasemanticanalyzer.h>
 #include <vala/valaclass.h>
 #include <vala/valareport.h>
@@ -46,7 +42,10 @@
 #include <vala/valastruct.h>
 #include <vala/valacodecontext.h>
 #include <vala/valavaluetype.h>
+#include <vala/valalocalvariable.h>
+#include <vala/valaexpression.h>
 #include <vala/valacreationmethod.h>
+#include <vala/valascope.h>
 #include <vala/valaobjecttype.h>
 #include <vala/valaobjecttypesymbol.h>
 #include <vala/valatypeparameter.h>
@@ -54,6 +53,7 @@
 #include <vala/valasignal.h>
 #include <vala/valavoidtype.h>
 #include <vala/valaarraytype.h>
+#include <vala/valacodevisitor.h>
 #include <vala/valabasicblock.h>
 
 
@@ -111,6 +111,7 @@ static void vala_method_set_entry_point (ValaMethod* self, gboolean value);
 static gpointer vala_method_parent_class = NULL;
 static void vala_method_finalize (ValaCodeNode* obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
+static gint _vala_array_length (gpointer array);
 static int _vala_strcmp0 (const char * str1, const char * str2);
 
 
@@ -228,7 +229,7 @@ static void vala_method_real_accept_children (ValaCodeNode* base, ValaCodeVisito
 		(_error_type_it == NULL) ? NULL : (_error_type_it = (gee_collection_object_unref (_error_type_it), NULL));
 	}
 	if (self->priv->_result_var != NULL) {
-		vala_code_node_accept ((ValaCodeNode*) vala_local_variable_get_variable_type (self->priv->_result_var), visitor);
+		vala_code_node_accept ((ValaCodeNode*) self->priv->_result_var, visitor);
 	}
 	{
 		GeeIterator* _precondition_it;
@@ -384,8 +385,8 @@ static void vala_method_process_ccode_attribute (ValaMethod* self, ValaAttribute
 			int filename_it;
 			_tmp1 = NULL;
 			filename_collection = _tmp1 = g_strsplit (val, ",", 0);
-			filename_collection_length1 = g_strv_length (_tmp1);
-			for (filename_it = 0; filename_it < g_strv_length (_tmp1); filename_it = filename_it + 1) {
+			filename_collection_length1 = _vala_array_length (_tmp1);
+			for (filename_it = 0; filename_it < _vala_array_length (_tmp1); filename_it = filename_it + 1) {
 				const char* _tmp2;
 				char* filename;
 				_tmp2 = NULL;
@@ -1142,7 +1143,7 @@ static gboolean vala_method_real_check (ValaCodeNode* base, ValaSemanticAnalyzer
 		(_error_type_it == NULL) ? NULL : (_error_type_it = (gee_collection_object_unref (_error_type_it), NULL));
 	}
 	if (self->priv->_result_var != NULL) {
-		vala_code_node_check ((ValaCodeNode*) vala_local_variable_get_variable_type (self->priv->_result_var), analyzer);
+		vala_code_node_check ((ValaCodeNode*) self->priv->_result_var, analyzer);
 	}
 	{
 		GeeIterator* _precondition_it;
@@ -1835,6 +1836,18 @@ static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify
 }
 
 
+static gint _vala_array_length (gpointer array) {
+	int length;
+	length = 0;
+	if (array) {
+		while (((gpointer*) array)[length]) {
+			length++;
+		}
+	}
+	return length;
+}
+
+
 static int _vala_strcmp0 (const char * str1, const char * str2) {
 	if (str1 == NULL) {
 		return -(str1 != str2);
