@@ -1,6 +1,7 @@
 /* valasemanticanalyzer.vala
  *
- * Copyright (C) 2006-2009  Jürg Billeter, Raffaele Sandrini
+ * Copyright (C) 2006-2009  Jürg Billeter
+ * Copyright (C) 2006-2008  Raffaele Sandrini
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,6 +33,12 @@
 #include <vala/valascope.h>
 #include <vala/valaclass.h>
 #include <vala/valadatatype.h>
+#include <vala/valacreationmethod.h>
+#include <vala/valamethod.h>
+#include <vala/valamember.h>
+#include <vala/valaconstructor.h>
+#include <vala/valadestructor.h>
+#include <vala/valaproperty.h>
 #include <vala/valagenerictype.h>
 #include <vala/valatypesymbol.h>
 #include <vala/valapointertype.h>
@@ -42,12 +49,6 @@
 #include <vala/valareport.h>
 #include <vala/valasourcereference.h>
 #include <vala/valastruct.h>
-#include <vala/valacreationmethod.h>
-#include <vala/valamethod.h>
-#include <vala/valamember.h>
-#include <vala/valaconstructor.h>
-#include <vala/valadestructor.h>
-#include <vala/valaproperty.h>
 #include <vala/valaexpression.h>
 #include <vala/valamethodtype.h>
 #include <vala/valaformalparameter.h>
@@ -369,50 +370,55 @@ ValaDataType* vala_semantic_analyzer_get_value_type_for_symbol (ValaSemanticAnal
 				ValaProperty* prop;
 				_tmp5 = NULL;
 				prop = (_tmp5 = VALA_PROPERTY (sym), (_tmp5 == NULL) ? NULL : vala_code_node_ref (_tmp5));
-				if (vala_property_get_property_type (prop) != NULL) {
-					ValaDataType* type;
-					ValaDataType* _tmp6;
-					type = vala_data_type_copy (vala_property_get_property_type (prop));
-					vala_data_type_set_value_owned (type, FALSE);
-					_tmp6 = NULL;
-					return (_tmp6 = type, (prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL)), _tmp6);
+				if (lvalue) {
+					if (vala_property_get_set_accessor (prop) != NULL) {
+						ValaDataType* _tmp6;
+						_tmp6 = NULL;
+						return (_tmp6 = vala_data_type_copy (vala_property_accessor_get_value_type (vala_property_get_set_accessor (prop))), (prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL)), _tmp6);
+					}
+				} else {
+					if (vala_property_get_get_accessor (prop) != NULL) {
+						ValaDataType* _tmp7;
+						_tmp7 = NULL;
+						return (_tmp7 = vala_data_type_copy (vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop))), (prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL)), _tmp7);
+					}
 				}
 				(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
 			} else {
 				if (VALA_IS_FORMAL_PARAMETER (sym)) {
-					ValaFormalParameter* _tmp7;
+					ValaFormalParameter* _tmp8;
 					ValaFormalParameter* p;
 					ValaDataType* type;
-					ValaDataType* _tmp8;
-					_tmp7 = NULL;
-					p = (_tmp7 = VALA_FORMAL_PARAMETER (sym), (_tmp7 == NULL) ? NULL : vala_code_node_ref (_tmp7));
+					ValaDataType* _tmp9;
+					_tmp8 = NULL;
+					p = (_tmp8 = VALA_FORMAL_PARAMETER (sym), (_tmp8 == NULL) ? NULL : vala_code_node_ref (_tmp8));
 					type = vala_data_type_copy (vala_formal_parameter_get_parameter_type (p));
 					if (!lvalue) {
 						vala_data_type_set_value_owned (type, FALSE);
 					}
-					_tmp8 = NULL;
-					return (_tmp8 = type, (p == NULL) ? NULL : (p = (vala_code_node_unref (p), NULL)), _tmp8);
+					_tmp9 = NULL;
+					return (_tmp9 = type, (p == NULL) ? NULL : (p = (vala_code_node_unref (p), NULL)), _tmp9);
 				} else {
 					if (VALA_IS_LOCAL_VARIABLE (sym)) {
-						ValaLocalVariable* _tmp9;
+						ValaLocalVariable* _tmp10;
 						ValaLocalVariable* local;
 						ValaDataType* type;
-						gboolean _tmp10;
-						ValaDataType* _tmp11;
-						_tmp9 = NULL;
-						local = (_tmp9 = VALA_LOCAL_VARIABLE (sym), (_tmp9 == NULL) ? NULL : vala_code_node_ref (_tmp9));
+						gboolean _tmp11;
+						ValaDataType* _tmp12;
+						_tmp10 = NULL;
+						local = (_tmp10 = VALA_LOCAL_VARIABLE (sym), (_tmp10 == NULL) ? NULL : vala_code_node_ref (_tmp10));
 						type = vala_data_type_copy (vala_local_variable_get_variable_type (local));
-						_tmp10 = FALSE;
+						_tmp11 = FALSE;
 						if (!lvalue) {
-							_tmp10 = !vala_local_variable_get_floating (local);
+							_tmp11 = !vala_local_variable_get_floating (local);
 						} else {
-							_tmp10 = FALSE;
+							_tmp11 = FALSE;
 						}
-						if (_tmp10) {
+						if (_tmp11) {
 							vala_data_type_set_value_owned (type, FALSE);
 						}
-						_tmp11 = NULL;
-						return (_tmp11 = type, (local == NULL) ? NULL : (local = (vala_code_node_unref (local), NULL)), _tmp11);
+						_tmp12 = NULL;
+						return (_tmp12 = type, (local == NULL) ? NULL : (local = (vala_code_node_unref (local), NULL)), _tmp12);
 					} else {
 						if (VALA_IS_ENUM_VALUE (sym)) {
 							return (ValaDataType*) vala_enum_value_type_new (VALA_ENUM (vala_symbol_get_parent_symbol (sym)));
@@ -989,54 +995,47 @@ static gboolean vala_semantic_analyzer_check_argument (ValaSemanticAnalyzer* sel
 						return FALSE;
 					} else {
 						gboolean _tmp14;
-						gboolean _tmp15;
 						_tmp14 = FALSE;
-						_tmp15 = FALSE;
-						if (vala_code_context_get_non_null (self->priv->_context)) {
-							_tmp15 = direction != VALA_PARAMETER_DIRECTION_OUT;
-						} else {
-							_tmp15 = FALSE;
-						}
-						if (_tmp15) {
+						if (direction != VALA_PARAMETER_DIRECTION_OUT) {
 							_tmp14 = !vala_data_type_get_nullable (vala_expression_get_target_type (arg));
 						} else {
 							_tmp14 = FALSE;
 						}
 						if (_tmp14) {
-							char* _tmp16;
-							_tmp16 = NULL;
-							vala_report_warning (vala_code_node_get_source_reference ((ValaCodeNode*) arg), _tmp16 = g_strdup_printf ("Argument %d: Cannot pass null to non-null parameter type", i + 1));
-							_tmp16 = (g_free (_tmp16), NULL);
+							char* _tmp15;
+							_tmp15 = NULL;
+							vala_report_warning (vala_code_node_get_source_reference ((ValaCodeNode*) arg), _tmp15 = g_strdup_printf ("Argument %d: Cannot pass null to non-null parameter type", i + 1));
+							_tmp15 = (g_free (_tmp15), NULL);
 						}
 					}
 				} else {
 					if (arg_type == 1) {
 						if (direction != VALA_PARAMETER_DIRECTION_IN) {
-							char* _tmp17;
-							_tmp17 = NULL;
-							vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) arg), _tmp17 = g_strdup_printf ("Argument %d: Cannot pass value to reference or output parameter", i + 1));
-							_tmp17 = (g_free (_tmp17), NULL);
+							char* _tmp16;
+							_tmp16 = NULL;
+							vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) arg), _tmp16 = g_strdup_printf ("Argument %d: Cannot pass value to reference or output parameter", i + 1));
+							_tmp16 = (g_free (_tmp16), NULL);
 							return FALSE;
 						}
 					} else {
 						if (arg_type == 2) {
 							if (direction != VALA_PARAMETER_DIRECTION_REF) {
-								char* _tmp19;
-								_tmp19 = NULL;
-								vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) arg), _tmp19 = g_strdup_printf ("Argument %d: Cannot pass ref argument to non-reference parameter", i + 1));
-								_tmp19 = (g_free (_tmp19), NULL);
+								char* _tmp18;
+								_tmp18 = NULL;
+								vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) arg), _tmp18 = g_strdup_printf ("Argument %d: Cannot pass ref argument to non-reference parameter", i + 1));
+								_tmp18 = (g_free (_tmp18), NULL);
 								return FALSE;
 							}
 							/* weak variables can only be used with weak ref parameters*/
 							if (vala_data_type_is_disposable (vala_expression_get_target_type (arg))) {
-								gboolean _tmp21;
-								_tmp21 = FALSE;
+								gboolean _tmp20;
+								_tmp20 = FALSE;
 								if (!(VALA_IS_POINTER_TYPE (vala_expression_get_value_type (arg)))) {
-									_tmp21 = !vala_data_type_get_value_owned (vala_expression_get_value_type (arg));
+									_tmp20 = !vala_data_type_get_value_owned (vala_expression_get_value_type (arg));
 								} else {
-									_tmp21 = FALSE;
+									_tmp20 = FALSE;
 								}
-								if (_tmp21) {
+								if (_tmp20) {
 									/* variable doesn't own the value */
 									vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) arg), "Invalid assignment from owned expression to unowned variable");
 									return FALSE;
@@ -1045,22 +1044,22 @@ static gboolean vala_semantic_analyzer_check_argument (ValaSemanticAnalyzer* sel
 						} else {
 							if (arg_type == 3) {
 								if (direction != VALA_PARAMETER_DIRECTION_OUT) {
-									char* _tmp23;
-									_tmp23 = NULL;
-									vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) arg), _tmp23 = g_strdup_printf ("Argument %d: Cannot pass out argument to non-output parameter", i + 1));
-									_tmp23 = (g_free (_tmp23), NULL);
+									char* _tmp22;
+									_tmp22 = NULL;
+									vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) arg), _tmp22 = g_strdup_printf ("Argument %d: Cannot pass out argument to non-output parameter", i + 1));
+									_tmp22 = (g_free (_tmp22), NULL);
 									return FALSE;
 								}
 								/* weak variables can only be used with weak out parameters*/
 								if (vala_data_type_is_disposable (vala_expression_get_target_type (arg))) {
-									gboolean _tmp25;
-									_tmp25 = FALSE;
+									gboolean _tmp24;
+									_tmp24 = FALSE;
 									if (!(VALA_IS_POINTER_TYPE (vala_expression_get_value_type (arg)))) {
-										_tmp25 = !vala_data_type_get_value_owned (vala_expression_get_value_type (arg));
+										_tmp24 = !vala_data_type_get_value_owned (vala_expression_get_value_type (arg));
 									} else {
-										_tmp25 = FALSE;
+										_tmp24 = FALSE;
 									}
-									if (_tmp25) {
+									if (_tmp24) {
 										/* variable doesn't own the value */
 										vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) arg), "Invalid assignment from owned expression to unowned variable");
 										return FALSE;
