@@ -1,6 +1,6 @@
 /* valagirparser.vala
  *
- * Copyright (C) 2008  Jürg Billeter
+ * Copyright (C) 2008-2009  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -119,6 +119,7 @@ static char* vala_gir_parser_get_attribute (ValaGirParser* self, const char* nod
 static gpointer vala_gir_parser_parent_class = NULL;
 static void vala_gir_parser_finalize (ValaCodeVisitor* obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
+static gint _vala_array_length (gpointer array);
 static int _vala_strcmp0 (const char * str1, const char * str2);
 
 
@@ -730,7 +731,7 @@ static ValaEnumValue* vala_gir_parser_parse_enumeration_member (ValaGirParser* s
 	_tmp1 = NULL;
 	_tmp0 = NULL;
 	_tmp5 = NULL;
-	ev = (_tmp5 = vala_enum_value_new (_tmp4 = g_strjoinv ("_", (_tmp3 = _tmp2 = g_strsplit (_tmp1 = g_utf8_strup (_tmp0 = vala_markup_reader_get_attribute (self->priv->reader, "name"), -1), "-", 0), _tmp3_length1 = g_strv_length (_tmp2), _tmp3)), NULL), _tmp4 = (g_free (_tmp4), NULL), _tmp3 = (_vala_array_free (_tmp3, _tmp3_length1, (GDestroyNotify) g_free), NULL), _tmp1 = (g_free (_tmp1), NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp5);
+	ev = (_tmp5 = vala_enum_value_new (_tmp4 = g_strjoinv ("_", (_tmp3 = _tmp2 = g_strsplit (_tmp1 = g_utf8_strup (_tmp0 = vala_markup_reader_get_attribute (self->priv->reader, "name"), -1), "-", 0), _tmp3_length1 = _vala_array_length (_tmp2), _tmp3)), NULL), _tmp4 = (g_free (_tmp4), NULL), _tmp3 = (_vala_array_free (_tmp3, _tmp3_length1, (GDestroyNotify) g_free), NULL), _tmp1 = (g_free (_tmp1), NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp5);
 	_tmp6 = NULL;
 	vala_enum_value_set_cname (ev, _tmp6 = vala_markup_reader_get_attribute (self->priv->reader, "c:identifier"));
 	_tmp6 = (g_free (_tmp6), NULL);
@@ -1013,7 +1014,7 @@ static ValaDataType* vala_gir_parser_parse_type_from_name (ValaGirParser* self,
 				}
 				_tmp7 = NULL;
 				_tmp6 = NULL;
-				type_components = (_tmp7 = _tmp6 = g_strsplit (type_name, ".", 0), type_components_length1 = g_strv_length (_tmp6), type_components_size = type_components_length1, _tmp7);
+				type_components = (_tmp7 = _tmp6 = g_strsplit (type_name, ".", 0), type_components_length1 = _vala_array_length (_tmp6), type_components_size = type_components_length1, _tmp7);
 				if (type_components[1] != NULL) {
 					char* namespace_name;
 					const char* _tmp8;
@@ -1536,15 +1537,15 @@ static ValaProperty* vala_gir_parser_parse_property (ValaGirParser* self) {
 	ValaSourceReference* _tmp4;
 	ValaProperty* _tmp5;
 	ValaProperty* prop;
-	gboolean _tmp7;
-	ValaProperty* _tmp9;
+	gboolean _tmp8;
+	ValaProperty* _tmp11;
 	g_return_val_if_fail (self != NULL, NULL);
 	vala_gir_parser_start_element (self, "property");
 	_tmp2 = NULL;
 	_tmp1 = NULL;
 	_tmp0 = NULL;
 	_tmp3 = NULL;
-	name = (_tmp3 = g_strjoinv ("_", (_tmp2 = _tmp1 = g_strsplit (_tmp0 = vala_markup_reader_get_attribute (self->priv->reader, "name"), "-", 0), _tmp2_length1 = g_strv_length (_tmp1), _tmp2)), _tmp2 = (_vala_array_free (_tmp2, _tmp2_length1, (GDestroyNotify) g_free), NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp3);
+	name = (_tmp3 = g_strjoinv ("_", (_tmp2 = _tmp1 = g_strsplit (_tmp0 = vala_markup_reader_get_attribute (self->priv->reader, "name"), "-", 0), _tmp2_length1 = _vala_array_length (_tmp1), _tmp2)), _tmp2 = (_vala_array_free (_tmp2, _tmp2_length1, (GDestroyNotify) g_free), NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp3);
 	readable = vala_markup_reader_get_attribute (self->priv->reader, "readable");
 	writable = vala_markup_reader_get_attribute (self->priv->reader, "writable");
 	construct_ = vala_markup_reader_get_attribute (self->priv->reader, "construct");
@@ -1555,26 +1556,32 @@ static ValaProperty* vala_gir_parser_parse_property (ValaGirParser* self) {
 	prop = (_tmp5 = vala_property_new (name, type, NULL, NULL, _tmp4 = vala_gir_parser_get_current_src (self)), (_tmp4 == NULL) ? NULL : (_tmp4 = (vala_source_reference_unref (_tmp4), NULL)), _tmp5);
 	vala_symbol_set_access ((ValaSymbol*) prop, VALA_SYMBOL_ACCESSIBILITY_PUBLIC);
 	if (_vala_strcmp0 (readable, "0") != 0) {
-		ValaPropertyAccessor* _tmp6;
+		ValaPropertyAccessor* _tmp7;
+		ValaDataType* _tmp6;
+		_tmp7 = NULL;
 		_tmp6 = NULL;
-		vala_property_set_get_accessor (prop, _tmp6 = vala_property_accessor_new (TRUE, FALSE, FALSE, NULL, NULL));
+		vala_property_set_get_accessor (prop, _tmp7 = vala_property_accessor_new (TRUE, FALSE, FALSE, _tmp6 = vala_data_type_copy (vala_property_get_property_type (prop)), NULL, NULL));
+		(_tmp7 == NULL) ? NULL : (_tmp7 = (vala_code_node_unref (_tmp7), NULL));
 		(_tmp6 == NULL) ? NULL : (_tmp6 = (vala_code_node_unref (_tmp6), NULL));
 	}
-	_tmp7 = FALSE;
+	_tmp8 = FALSE;
 	if (_vala_strcmp0 (writable, "1") == 0) {
-		_tmp7 = TRUE;
+		_tmp8 = TRUE;
 	} else {
-		_tmp7 = _vala_strcmp0 (construct_, "1") == 0;
+		_tmp8 = _vala_strcmp0 (construct_, "1") == 0;
 	}
-	if (_tmp7) {
-		ValaPropertyAccessor* _tmp8;
-		_tmp8 = NULL;
-		vala_property_set_set_accessor (prop, _tmp8 = vala_property_accessor_new (FALSE, (_vala_strcmp0 (writable, "1") == 0), (_vala_strcmp0 (construct_, "1") == 0), NULL, NULL));
-		(_tmp8 == NULL) ? NULL : (_tmp8 = (vala_code_node_unref (_tmp8), NULL));
+	if (_tmp8) {
+		ValaPropertyAccessor* _tmp10;
+		ValaDataType* _tmp9;
+		_tmp10 = NULL;
+		_tmp9 = NULL;
+		vala_property_set_set_accessor (prop, _tmp10 = vala_property_accessor_new (FALSE, (_vala_strcmp0 (writable, "1") == 0), (_vala_strcmp0 (construct_, "1") == 0), _tmp9 = vala_data_type_copy (vala_property_get_property_type (prop)), NULL, NULL));
+		(_tmp10 == NULL) ? NULL : (_tmp10 = (vala_code_node_unref (_tmp10), NULL));
+		(_tmp9 == NULL) ? NULL : (_tmp9 = (vala_code_node_unref (_tmp9), NULL));
 	}
 	vala_gir_parser_end_element (self, "property");
-	_tmp9 = NULL;
-	return (_tmp9 = prop, name = (g_free (name), NULL), readable = (g_free (readable), NULL), writable = (g_free (writable), NULL), construct_ = (g_free (construct_), NULL), (type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL)), _tmp9);
+	_tmp11 = NULL;
+	return (_tmp11 = prop, name = (g_free (name), NULL), readable = (g_free (readable), NULL), writable = (g_free (writable), NULL), construct_ = (g_free (construct_), NULL), (type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL)), _tmp11);
 }
 
 
@@ -1834,7 +1841,7 @@ static ValaSignal* vala_gir_parser_parse_signal (ValaGirParser* self) {
 	_tmp1 = NULL;
 	_tmp0 = NULL;
 	_tmp3 = NULL;
-	name = (_tmp3 = g_strjoinv ("_", (_tmp2 = _tmp1 = g_strsplit (_tmp0 = vala_markup_reader_get_attribute (self->priv->reader, "name"), "-", 0), _tmp2_length1 = g_strv_length (_tmp1), _tmp2)), _tmp2 = (_vala_array_free (_tmp2, _tmp2_length1, (GDestroyNotify) g_free), NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp3);
+	name = (_tmp3 = g_strjoinv ("_", (_tmp2 = _tmp1 = g_strsplit (_tmp0 = vala_markup_reader_get_attribute (self->priv->reader, "name"), "-", 0), _tmp2_length1 = _vala_array_length (_tmp1), _tmp2)), _tmp2 = (_vala_array_free (_tmp2, _tmp2_length1, (GDestroyNotify) g_free), NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp3);
 	vala_gir_parser_next (self);
 	return_type = NULL;
 	_tmp4 = FALSE;
@@ -2026,8 +2033,8 @@ void vala_gir_parser_parse_metadata (ValaGirParser* self, const char* metadata_f
 				int line_it;
 				_tmp3 = NULL;
 				line_collection = _tmp3 = g_strsplit (metadata, "\n", 0);
-				line_collection_length1 = g_strv_length (_tmp3);
-				for (line_it = 0; line_it < g_strv_length (_tmp3); line_it = line_it + 1) {
+				line_collection_length1 = _vala_array_length (_tmp3);
+				for (line_it = 0; line_it < _vala_array_length (_tmp3); line_it = line_it + 1) {
 					const char* _tmp12;
 					char* line;
 					_tmp12 = NULL;
@@ -2045,7 +2052,7 @@ void vala_gir_parser_parse_metadata (ValaGirParser* self, const char* metadata_f
 						}
 						_tmp5 = NULL;
 						_tmp4 = NULL;
-						tokens = (_tmp5 = _tmp4 = g_strsplit (line, " ", 2), tokens_length1 = g_strv_length (_tmp4), tokens_size = tokens_length1, _tmp5);
+						tokens = (_tmp5 = _tmp4 = g_strsplit (line, " ", 2), tokens_length1 = _vala_array_length (_tmp4), tokens_size = tokens_length1, _tmp5);
 						if (NULL == tokens[0]) {
 							line = (g_free (line), NULL);
 							tokens = (_vala_array_free (tokens, tokens_length1, (GDestroyNotify) g_free), NULL);
@@ -2058,8 +2065,8 @@ void vala_gir_parser_parse_metadata (ValaGirParser* self, const char* metadata_f
 							int attribute_it;
 							_tmp6 = NULL;
 							attribute_collection = _tmp6 = g_strsplit (tokens[1], " ", 0);
-							attribute_collection_length1 = g_strv_length (_tmp6);
-							for (attribute_it = 0; attribute_it < g_strv_length (_tmp6); attribute_it = attribute_it + 1) {
+							attribute_collection_length1 = _vala_array_length (_tmp6);
+							for (attribute_it = 0; attribute_it < _vala_array_length (_tmp6); attribute_it = attribute_it + 1) {
 								const char* _tmp11;
 								char* attribute;
 								_tmp11 = NULL;
@@ -2075,7 +2082,7 @@ void vala_gir_parser_parse_metadata (ValaGirParser* self, const char* metadata_f
 									char* _tmp10;
 									_tmp8 = NULL;
 									_tmp7 = NULL;
-									pair = (_tmp8 = _tmp7 = g_strsplit (attribute, "=", 2), pair_length1 = g_strv_length (_tmp7), pair_size = pair_length1, _tmp8);
+									pair = (_tmp8 = _tmp7 = g_strsplit (attribute, "=", 2), pair_length1 = _vala_array_length (_tmp7), pair_size = pair_length1, _tmp8);
 									_tmp9 = FALSE;
 									if (pair[0] == NULL) {
 										_tmp9 = TRUE;
@@ -2211,6 +2218,18 @@ static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify
 }
 
 
+static gint _vala_array_length (gpointer array) {
+	int length;
+	length = 0;
+	if (array) {
+		while (((gpointer*) array)[length]) {
+			length++;
+		}
+	}
+	return length;
+}
+
+
 static int _vala_strcmp0 (const char * str1, const char * str2) {
 	if (str1 == NULL) {
 		return -(str1 != str2);
